.TH XWCREATETILE 3X
.SH NAME
XwCreateTile \- create a tile suitable for area filling or
patterned text.
.SH SYNOPSIS
#include <X11/Intrinsic.h>
.br
#include <Xw/Xw.h>
.PP
Pixmap XwCreateTile (screen, foreground, background,
tileType)
.br
Screen * screen;
.br
Pixel foreground;
.br
Pixel background;
.br
int tileType;
.SH ARGUMENTS
.TP 5
screen
.br
This parameter contains the screen for which the tile
is to be created.
.TP 5
foreground
.br
This is the foreground color to use for creating the
tile.
.TP 5
background
.br
This is the background color to use for creating the
tile.
.TP 5
tileType
.br
This is an integer value representing a particular
pattern to use when creating the tile.
.SH DESCRIPTION
XwCreateTile is a function (not a widget) that creates and
returns a pixmap of screen depth, using the foreground and
background colors specified.  The tileType parameter is used
to select the particular tile to create.  Duplicate requests
for the same tile, screen, foreground and background are
cached to reduce overhead.
.PP
There are nine available tile types.  They are defined by a
set of #define statements in the file Xw.h and are described
in the following table.
.ti -5
.PP
Define              Description
.br
.ti -5
________________________________________________________________
.br
.ti -5
XwFOREGROUND        A tile of solid foreground
.br
.ti -5
XwBACKGROUND        A tile of solid background
.br
.ti -5
Xw25_FOREGROUND     A tile of 25% foreground, 75% background
.br
.ti -5
.PP
Xw50_FOREGROUND     A tile of 50% foreground, 50% background
.br
.ti -5
Xw75_FOREGROUND     A tile of 75% foreground, 25% background
.br
.ti -5
XwHORIZONTAL_TILE   A tile of horizontal lines of the two colors
.br
.ti -5
XwVERTICAL_TILE     A tile of vertical lines of the two colors
.br
.ti -5
XwSLANT_RIGHT       A tile of slanting lines of the two colors
.br
.ti -5
XwSLANT_LEFT        A tile of slanting lines of the two colors
.br
.ti -5
.PP
To use a tile created by this function, the returned tile should
.br
.ti -5
be placed into the tile field of a graphics context, and the
.br
.ti -5
fill_style should be set to FillTiled.
.br
.SH RESOURCES
XwCreateTile gives the application or widget writer an easy
mechanism to specify the tile type to use.  The tile type
can be specified within the .Xdefaults file or an argument
list.  A resource converter is present to convert .Xdefault
strings into the matching defined value for each of the
tiles.  The strings to be contained within the .Xdefaults
file are as follows.
.IP "" 10
Xdefault String   Define
___________________________________
.br
foreground        XwFOREGROUND
.br
background        XwBACKGROUND
.br
25_foreground     Xw25_FOREGROUND
.br
50_foreground     Xw50_FOREGROUND
.br
75_foreground     Xw75_FOREGROUND
.br
horizontal_tile   XwHORIZONTAL_TILE
.br
vertical_tile     XwVERTICAL_TILE
.br
slant_right       XwSLANT_RIGHT
.br
slant_left        XwSLANT_LEFT
.ti -5
.PP
For widget writers who wish to incorporate settable tiles within
.br
.ti -5
their resource set, the representation member of the resource
.br
.ti -5
definition should be set to the define XtRTileType.
.br
.SH "RETURN VALUES"
XwCREATETILE returns a pixmap if successful.  If an invalid
tile type or screen is specified, 0 is returned.
.SH ORIGIN
Hewlett-Packard Company.
.SH "SEE ALSO"
Programming With The HP X Widgets,
.br
Programming With Xt Intrinsics,
.br
Programming With Xlib.
