.TH XWTITLEBAR 3X
.SH NAME
XwtitlebarWidgetClass \- An X Widget for creating titlebars.
.SH SYNOPSIS
#include <X11/StringDefs.h>
.br
#include <X11/Intrinsic.h>
.br
#include <Xw/Xw.h>
.br
#include <Xw/TitleBar.h>
.SH CLASSES
A subclass of Core, Composite, Constraint and
XwManagerClass.
.PP
The widget class to use when creating a TitleBar widget is
XwtitlebarWidgetClass.
.PP
The class name of TitleBar is TitleBar.
.SH DESCRIPTION
TitleBar provides a flexible mechanism for creating
titlebars containing text and arbitrary widgets.  Inputs are
an optional text string and any number of widgets to manage.
The title string will be displayed in a StaticText widget
(refer to XWSTATICTEXT(3X)).  Managed widgets may have
optionally specified layout information (see CONSTRAINT
RESOURCES below).
.PP
When TitleBar is directed to become narrower than is
neccessary to display all of its interior widgets, some
widgets may be hidden.  The XtNprecedence resource in each
managed widget controls this feature.
.PP
As TitleBar is directed to become narrower and narrower,
widgets whose sum of XtNrPadding and XtNlPadding is greater
than zero will have their padding collapsed to one pixel.
Widgets will have their padding stripped in order of
decreasing values of XtNprecedence.
.PP
If, after collapsing all of the widgets' padding, TitleBar
is still too narrow to display all of its children widgets,
widgets will be hidden.  Widgets will be hidden in order of
decreasing values of XtNprecedence.  TitleBar will try to
always display a widget of the highest priority (lowest
value of XtNprecedence, even if it must be clipped.
.PP
Users of TitleBar should note that when children widgets are
hidden they are completely hidden.  Additionally, users who
wish to make extensive use of the obscurability rules should
read carefully the section on XtNprececence in the
CONSTRAINT RESOURCES section below.
.SH "NEW RESOURCES"
To specify any of these resources within a resource defaults
file, simply drop the XtN prefix from the resource name.
TitleBar defines the following new resources:
.IP "" 20
TitleBar Resource Set
.ti -25
Name                  Class                Type          Default
.br
.ti -25
_______________________________________________________________________
.br
.ti -25
XtNtitlePrecedence    XtCTitlePrecedence   int           0
.br
.ti -25
XtNtitleRegion        XtCTitleRegion       XwAlignment   XwALIGN_CENTER
.br
.ti -25
XtNtitlePosition      XtCTitlePosition     int           0
.br
.ti -25
XtNtitleRPadding      XtCTitleRPadding     int           1
.br
.ti -25
XtNtitleLPadding      XtCTitleLPadding     int           1
.br
.ti -25
XtNtitleForeground    XtCForeground        Pixel         black
.br
.ti -25
XtNtitleBackground    XtCBackground        Pixel         white
.br
.ti -25
XtNtitleHSpace        XtNTitleHSpace       int           2
.br
.ti -25
XtNtitleVSpace        XtVTitleHSpace       int           2
.br
.ti -25
XtNtitleBorderWidth   XtCBorderWidth       int           0
.br
.ti -25
XtNtitleSelect        XtCCallback          Pointer       NULL
.br
.ti -25
XtNtitleRelease       XtCCallback          Pointer       NULL
.br
.ti -25
XtNhSpace             XtCHSpace            int           2
.br
.ti -25
XtNvSpace             XtCVSpace            int           2
.br
.ti -25
XtNenter              XtCCallback          Pointer       NULL
.br
.ti -25
XtNleave              XtCCallback          Pointer       NULL
.br
.ti -25
XtNselect             XtCCallback          Pointer       NULL
.br
.ti -25
XtNrelease            XtCCallback          Pointer       NULL
.br
.TP 5
XtNtitlePrecedence
.br
The value to be loaded into the constraint record of
the optional StaticText widget.
.TP 5
XtNtitleRegion
.br
The value to be loaded into the XtNtitleRegion
constraint resource of the optional StaticText widget.
.TP 5
XtNtitlePosition
.br
The value to be loaded into the XtNtitlePosition
constraint resource of the optional StaticText widget.
.TP 5
XtNtitleRPadding
.br
The value to be loaded into the XtNtitleRPadding
constraint resource of the optional StaticText widget.
.TP 5
XtNtitleLPadding
.br
The value to be loaded into the XtNtitleLPadding
constraint resource of the optional StaticText widget.
.PP
XtNtitleForeground
.IP "" 5
The value to be loaded into the XtNforeground resource
of the optional StaticText widget's core part.
.TP 5
XtNtitleBackground
.br
The value to be loaded into the XtNbackground resource
of the optional StaticText widget's core part.
.TP 5
XtNtitleHSpace
.br
The value to be loaded into the XtNhSpace resource of
the optional StaticText widget.
.TP 5
XtNtitleVSpace
.br
The value to be loaded into the XtNvSpace resource of
the optional StaticText widget.
.TP 5
XtNtitleBorderWidth
.br
The value to loaded into the XtNborderWidth resource of
the optional StaticText widget.
.TP 5
XtNtitleSelect
.br
The value loaded into the XtNselect resource of the
optional StaticText widget.
.TP 5
XtNtitleRelease
.br
The value loaded into the XtNrelease resource of the
optional StaticText widget.
.TP 5
XtNhSpace
.br
The amount of space to maintain between the right and
left of the titlebar and the interior widgets.
.TP 5
XtNvSpace
.br
The amount of space to maintain between the top and
bottom of the titlebar and the interior widgets.
.TP 5
XtNenter, XtNleave, XtNselect, and XtNrelease
.br
Callbacks provided for control of TitleBar.  The data
parameter is unused.
.TP 0
.ti -5
INCORPORATED RESOURCES
.br
The TitleBar creates an internal StaticText widget to handle
the title string.  In order to provide the user some control
over the appearance of this internal widget, the following
resources defined by StaticText are incorporated into
TitleBar's resource list.
.PP
It must be noted that only the resources within the
following tables will have any effect on the internal
StaticText widget.  The other resources defined for
StaticText will be overridden by TitleBar.
.PP
For a complete description of the following resources, refer
to the manual page given in the table heading.
.IP "" 10
Primitive Resource Set \-\- XWPRIMITIVE(3X)
.ti -15
Name                    Class                   Type    Default
.br
.ti -15
________________________________________________________________________
.br
.ti -15
XtNhighlightColor       XtCForeground           Pixel   Black
.br
.ti -15
XtNhighlightStyle       XtCHighlightStyle       int     XwPATTERN_BORDER
.br
.ti -15
XtNhighlightTile        XtCHighlightTile        int     XwBACKGROUND
.br
.ti -15
XtNhighlightThickness   XtCHighlightThickness   int     0
.br
.ti -15
XtNtraversalType        XtCTraversalType        int     HIGHLIGHT_OFF
.br
.IP "" 6
Static Text Resource Set \-\- STATICTEXT(3X)
.ti -9
Name           Class          Type            Default
.br
.ti -9
____________________________________________________________
.br
.ti -9
XtNstring      XtCString      char *          NULL
.br
.ti -9
XtNalignment   XtCAlignment   XwAlignment     XwALIGN_CENTER
.br
.ti -9
XtNwrap        XwCWrap        Boolean         TRUE
.br
.ti -9
XtNlineSpace   XtCLineSpace   int             0
.br
.ti -9
XtNfont        XtCFont        XFontStruct *   Fixed
.br
.SH "CONSTRAINT RESOURCES"
The following resources will be attached to every widget
inserted into TitleBar.  See CONSTRAINT(3X) for a general
discussion of constraint resources.
.PP
TitleBar uses the constraint resources as hints during the
layout of its managed children.  Under certain conditions,
any of these resources except XtNprecedence can (and will)
be ignored by TitleBar.
.IP "" 1
Constraint Resource Set \-\- Children of TITLEBAR(3X)
.ti -2
Name            Class           Type          Default
.br
.ti -2
________________________________________________________
.br
.ti -2
XtNregion       XtCRegion       XwAlignment   See below.
.br
.ti -2
XtNposition     XtCPosition     int           0
.br
.ti -2
XtNlPadding     XtCLPadding     int           2
.br
.ti -2
XtNrPadding     XtCRPadding     int           2
.br
.ti -2
XtNprecedence   XtCPrecedence   int           1
.br
.TP 5
XtNregion
.br
Associates a child with a region of the titlebar.  The
regions may be specified in the resource default file
as "left" for XwALIGN_LEFT, "center" for
XwALIGN_CENTER, and "right" for XwALIGN_RIGHT.
.IP "" 5
During layout widgets with XtNregion values of
XwALIGN_LEFT grouped to the left end of TitleBar.
Widgets with XtNregion values of XwALIGN_LEFT are
grouped to the right of TitleBar.  Widgets with
XtNregion values of XwALIGN_CENTER will be grouped
between the left and right groups.  Additionally,
TitleBar tries to center the center group within the
TitleBar.
.IP "" 5
Widgets for which XtNregion is unspecified or
XwALIGN_NONE when XtNstring is non-null, will be
.IP "" 5
assigned one of the two regions not equal to
XtNtitleRegion in an alternating fashion.
.IP "" 5
Widgets for which XtNregion is unspecified or
XwALIGN_NONE when XtNstring is null, will be assigned a
region.  The first such widget will be assigned to the
left region, the next to the center region, the next to
the right region, the next to the left region, and so
forth.
.TP 5
XtNposition
.br
This resource gives the order of widgets within region.
The left and the center region are layed out with
XtNposition values increasing from left to right.  The
right region is laid out with XtNposition values
increasing from right to left.
.IP "" 5
Position values are unique within a region.  If two
widgets are assigned the same position, the widget
which was assigned first gets the position.  The second
widget gets the next available position.  For example,
widget1 and widget2 are the only widgets inserted in
TitleBar.  Widget1 is inserted befor widget2.  Widget1
and widget2 are both assigned a position of 4.  Widget1
will be given the position of 4, and widget2 will be
assigned a position of 5.
.TP 5
XtNlPadding
.br
The number of pixels that TitleBar should try to
maintain between the left of the widget and the right
padding of the sibling widget to the left.  For
example, widget1 is to the left of widget2 within
TitleBar.  Widget1 has a XtNrPadding value of 5.
Widget2 has a XtNlPadding value of 5.  The borders of
widget1 and widget2 will be 10 pixels apart.
.IP "" 5
If TitleBar is too narrow to honor all of its
children's padding requests without hiding some
children, some, possibly all, padding requests will be
collapsed.
.TP 5
XtNrPadding
.br
The number of pixels that TitleBar should try to
maintain between the right of the widget and the left
padding of the sibling widget to the right.  For
example, widget1 is to the right of widget2 within
TitleBar.  Widget1 has a XtNlPadding value of 5.
Widget2 has a XtNrPadding value of 5.  The borders of
widget1 and widget2 will be 10 pixels apart.
.IP "" 5
If TitleBar is too narrow to honor all of its
.IP "" 5
children's padding requests without hiding some
children, some, possibly all, padding requests will be
collapsed.
.TP 5
XtNprecedence
.br
When TitleBar is too narrow to display all of its
children, this resource is used to determine which
children should be hidden.  Widgets with high values of
XtNprecedence are hidden first.  Precedence values are
relative to all other widgets within an instantiation
of TitleBar.  This means that all widgets, regardless
of their region, with high values of XtNprecedence will
be hidden before any widgets with the next lower values
are hidden.
.IP "" 5
Values of XtNprecedence need not be unique.  If values
are unique, there is no question about which widget is
first to lose its padding, nor about which widget is
first to be hidden.
.IP "" 5
If values are not unique for all children of TitleBar,
there need be no question about which widget is acted
on first, but it is dependent on both insertion order
and precedence.  The last widget inserted in TitleBar
of a given precedence is the first to lose its
requested padding  (of widgets with that priority).
Widgets lose padding from last inserted to first
inserted, within a given level of precedence.  When
hiding widgets, widgets within a given precedence level
are hidden from last inserted to first inserted.
.SH "INHERITED RESOURCES"
The following resources are inherited from the indicated
superclasses:
.IP "" 17
Core Resource Set \-\- CORE(3X)
.ti -22
Name                   Class                  Type             Default
.br
.ti -22
__________________________________________________________________________
.br
.ti -22
XtNancestorSensitive   XtCSenstitive          Boolean          TRUE
.br
.ti -22
XtNx                   XtCPosition            int              0
.br
.ti -22
XtNy                   XtCPosition            int              0
.br
.ti -22
XtNwidth               XtCWidth               int              0
.br
.ti -22
XtNheight              XtCHeight              int              0
.br
.ti -22
XtNdepth               XtCDepth               int              0
.br
.ti -22
XtNbackground          XtCBackground          Pixel            White
.br
.ti -22
XtNbackgroundPixmap    XtCPixmap              Pixmap           Unspecified
.br
.ti -22
XtNborderWidth         XtCBorderWidth         int              1
.br
.ti -22
XtNborderColor         XtCBorderColor         Pixel            Black
.br
.ti -22
XtNborderPixmap        XtCPixmap              Pixmap           Unspecified
.br
.ti -22
XtNsensitive           XtCSensitive           Boolean          TRUE
.br
.ti -22
XtNmappedWhenManaged   XtCMappedWhenManaged   Boolean          TRUE
.br
.ti -22
XtNdestroyCallback     XtCCallback            Pointer          NULL
.br
.ti -22
XtNtranslations        XtCTranslations        XtTranslations   NULL
.br
.PP
The input to the toggle is driven by the mouse buttons.  The
default translation set defining this button is listed
below.
.br
.ti +17
<EnterWindow>:      enter()"
.br
.ti +17
<LeaveWindow>:      leave()"
.br
.ti +17
<Btn1Down>:         select()"
.br
.ti +17
<Btn1Up>:           release()"
.br
.SH ACTIONS
.TP 5
enter
.br
If keyboard traversal is active (argument type
XtNtraversalOn with argument value TRUE) and the parent
of this widget is not a subclass of XwManager, initiate
keyboard traversal.  After this, the callback list is
invoked.
.TP 5
leave
.br
If keyboard traversal is active (argument type
XtNtraversalOn with argument value TRUE) and the parent
of this widget is not a subclass of XwManager,
terminate keyboard traversal.  After this, the callback
list is invoked.
.PP
select
.IP "" 5
Invokes the select callback list.
.TP 5
release
.br
Invokes the release callback list.
.SH ORIGIN
Hewlett-Packard Company.
.SH "SEE ALSO"
CORE(3X), CONSTRAINT(3X), XWMANAGER(3X), XWSTATICTEXT(3X),
XWCREATETILE(3X),
.br
Programming With The HP X Widgets,
.br
Programming With Xt Intrinsics,
.br
Programming With Xlib.
