.TH XWVALUATOR 3X
.SH NAME
XwvaluatorWidgetClass \- the X Widget's valuator widget
.SH SYNOPSIS
#include <X11/StringDefs.h>
.br
#include <X11/Intrinsic.h>
.br
#include <Xw/Xw.h>
.br
#include <Xw/Valuator.h>
.SH CLASSES
The Valuator widget is built from the Core and XwPrimitive
classes.
.PP
The widget class to use when creating a valuator is
XwvaluatorWidgetClass.  The class name for Valuator is
Valuator.
.SH DESCRIPTION
The Valuator widget implements a horizontal or vertical
scrolling widget as a rectangular bar containing a sliding
box (slider).  The Valuator widget supports input through
interactive slider movement and selections on the slide area
not occupied by the slider.  Both types of input have a
separate callback list for communicating with the
application.  The Valuator widget can be used by the
application to attach to objects scrolled under application
control, or used by composite widgets to implement
predefined scrolled objects.
.SH "NEW RESOURCES"
The Valuator widget defines a set of resource types used by
the programmer to specify the data for the valuator.  The
programmer can also set the values for the Core and
Primitive widget classes to set attributes for this widget.
To reference a resource in a .Xdefaults file, strip off the
XtN from the resource string.  The following table contains
the set of resources defined by Valuator.
.IP "" 16
Valuator Resource Set
.ti -21
Name                  Class                 Type      Default
.br
.ti -21
________________________________________________________________
.br
.ti -21
XtNsliderMin          XtCSliderMin          int       0
.br
.ti -21
XtNsliderMax          XtCSliderMax          int       100
.br
.ti -21
XtNsliderExtent       XtCSliderExtent       int       10
.br
.ti -21
XtNsliderOrigin       XtCSliderOrigin       int       0
.br
.ti -21
XtNsliderTile         XtCSliderTile         int       foreground
.br
.ti -21
XtNslideOrientation   XtCSlideOrientation   int       vertical
.br
.ti -21
XtNsliderMoved        XtCCallback           Pointer   NULL
.br
.ti -21
XtNsliderReleased     XtCCallback           Pointer   NULL
.br
.ti -21
XtNareaSelected       XtCCallback           Pointer   NULL
.br
.TP 5
XtNsliderMin, XtNsliderMax
.br
The Valuator widget lets the application define its own
coordinate system for the valuator.  Any integer values
with sliderMin less than sliderMax can be specified.
.TP 5
XtNsliderExtent
.br
The size of the slider can be set by the application.
The acceptable values are 0 < sliderExtent < (sliderMax
\- sliderMin).
.TP 5
XtNsliderOrigin
.br
The location of the slider can be set by the
application.  The acceptable values are between
sliderMin and (sliderMax \- sliderExtent).
.TP 5
XtNsliderTile
.br
This resource is used to set the tile used to create
the pixmap to use when drawing the slider.  The
#defines for setting the values through an arg list and
the strings to be used in the .Xdefaults files are
described in XwCreateTile(3X).
.TP 5
XtNslideOrientation
.br
The Valuator widget supports both horizontal and
vertical scrolling.  This resource type is the means by
which this is set.  It can be defined through the
\&.Xdefaults file by the strings "horizontal", and
"vertical" or within an arg list for use in
XtSetValues() by the defines XwHORIZONTAL and
XwVERTICAL.
.TP 5
XtNsliderMoved, XtNsliderReleased, XtNareaSelected
.br
The Valuator widget defines three types of callback
lists which get invoked upon different event conditions
when interacting with a valuator. All types use the
data parameter to send the location of the slider to
.IP "" 5
the callback functions.
.IP "" 5
The first callback type, sliderMoved, defines the
callback list containing the callback functions called
when the slider is interactively moved.
.IP "" 5
The second callback type, sliderReleased, defines a
callback list containing callback functions called when
the mouse button is released after moving the slider.
.IP "" 5
The third callback type, areaSelected, defines a
callback list containing the callback functions called
when an area in a valuator not containing the slider is
selected.  The slider is not moved to this position but
if the application wants the slider moved, it can use
the position value contained in the parameter call_data
and perform a XtSetValues() to its valuator.
.IP "" 5
For the callback types, the call_data parameter of the
callback function will be an integer containing the
slider or selection position.
.SH "INHERITED RESOURCES"
The following resources are inherited from the named
superclasses:
.IP "" 17
Core Resource Set \-\- CORE(3X)
.ti -22
Name                   Class                  Type             Default
.br
.ti -22
__________________________________________________________________________
.br
.ti -22
XtNancestorSensitive   XtCSenstitive          Boolean          TRUE
.br
.ti -22
XtNx                   XtCPosition            int              0
.br
.ti -22
XtNy                   XtCPosition            int              0
.br
.ti -22
XtNwidth               XtCWidth               int              0
.br
.ti -22
XtNheight              XtCHeight              int              0
.br
.ti -22
XtNdepth               XtCDepth               int              0
.br
.ti -22
XtNbackground          XtCBackground          Pixel            White
.br
.ti -22
XtNbackgroundPixmap    XtCPixmap              Pixmap           Unspecified
.br
.ti -22
XtNborderWidth         XtCBorderWidth         int              1
.br
.ti -22
XtNborderColor         XtCBorderColor         Pixel            Black
.br
.ti -22
XtNborderPixmap        XtCPixmap              Pixmap           Unspecified
.br
.ti -22
XtNsensitive           XtCSensitive           Boolean          TRUE
.br
.ti -22
XtNmappedWhenManaged   XtCMappedWhenManaged   Boolean          TRUE
.br
.ti -22
XtNdestroyCallback     XtCCallback            Pointer          NULL
.br
.ti -22
XtNtranslations        XtCTranslations        XtTranslations   NULL
.br
.IP "" 10
Primitive Resource Set \-\- XWPRIMITIVE(3X)
.ti -15
Name                    Class                   Type      Default
.br
.ti -15
________________________________________________________________________
.br
.ti -15
XtNforeground           XtCForeground           Pixel     Black
.br
.ti -15
XtNbackgroundTile       XtCBackgroundTile       int       background
.br
.ti -15
XtNtraversalType        XtCTraversalType        int       highlight_off
.br
.ti -15
XtNhighlightStyle       XtCHighlightStyle       int       pattern_border
.br
.ti -15
XtNhighlightColor       XtCForeground           Pixel     Black
.br
.ti -15
XtNhighlightTile        XtCHighlightTile        int       50_foreground
.br
.ti -15
XtNhighlightThickness   XtCHighlightThickness   int       0
.br
.ti -15
XtNrecomputeSize        XtCRecomputeSize        Boolean   TRUE
.br
.ti -15
XtNselect               XtCCallback             Pointer   NULL
.br
.ti -15
XtNrelease              XtCCallback             Pointer   NULL
.br
.SH "KEYBOARD TRAVERSAL"
If the XtNtraversalType resource is set to
highlight_traversal (XwHIGHLIGHT_TRAVERSAL in an argument
list) at either create time or during a call to XtSetValues,
the XwPrimitive superclass will automatically augment the
primitive widget's translations to support keyboard
traversal.  See the XwPrimitive man page for a complete
description of these translations.  See the TRANSLATIONS
section in this man page for a description of the
translations local this widget.
.SH TRANSLATIONS
The input to the Valuator widget is driven by the mouse
buttons.  The default translation is defined as follows:
.ti -4
.PP
<Btn1Down>:          select(),
.br
.ti -4
<Btn1Up>:            release(),
.br
.ti -4
Button1<PtrMoved>:   moved(),
.br
.ti -4
<EnterWindow>:       enter(),
.br
.ti -4
<LeaveWindow>:       leave(),
.br
.ti -4
Ctrl<Key>Left:       left(),      HP "Control Left Cursor" key
.br
.ti -4
Ctrl<Key>Up:         up(),        HP "Control Up Cursor" key
.br
.ti -4
Ctrl<Key>Right:      right(),     HP "Control Right Cursor" key
.br
.ti -4
Ctrl<Key>Down:       down(),      HP "Control Down Cursor" key
.br
.SH ACTIONS
.TP 5
select:
.br
Select processes the activation conditions within the
valuator, both for selections within the slider area
and on the slider.
.TP 5
release:
.br
Release handles the processing terminating conditions
.IP "" 5
for selections on the valuator.
.TP 5
moved:
.br
Moved processes interactive movement of the slider
following a selection upon the slider.
.TP 5
enter:
.br
If the XtNtraversalType resource has been set to
XwHIGHLIGHT_ENTER then the arrow's border will be
highlighted.  Otherwise no action is taken.
.TP 5
leave:
.br
If the XtNtraversalType resource has been set to
XwHIGHLIGHT_ENTER then the arrow's border will be
unhighlighted.  Otherwise no action is taken.
.TP 5
left:
.br
If the valuator's orientation is horizontal, this
action will cause its slider origin to be decremented
by 1 unit and redisplayed.
.TP 5
up:
If the valuator's orientation is vertical, this action
will cause its slider origin to be decremented by 1
unit and redisplayed.
.TP 5
right:
.br
If the valuator's orientation is horizontal, this
action will cause its slider origin to be incremented
by 1 unit and redisplayed.
.TP 5
down:
.br
If the valuator's orientation is vertical, this action
will cause its slider origin to be incremented by 1
unit and redisplayed.
.SH ORIGIN
Hewlett-Packard Company.
.SH "SEE ALSO"
CORE(3X), XWPRIMITIVE(3X), XWCREATETILE(3X),
.br
Programming With The HP X Widgets,
.br
Programming With Xt Intrinsics,
.br
Programming With Xlib.
